/********************************************
 * serialport.c
 * Funktionen für den Datentransfer PC<->MCU 
 * über serielle Schnitstelle
 * ehemals pc<->mcu.c
 ********************************************/
#include <stdio.h>
#include <linux/ioctl.h>
#include <sys/ioctl.h>
#include "serialport.h"

/*Serielle Schnittstelle offnen*/
int serial_init(char *dev,int baud,struct serial_cfg *sr_cfg)
{
    int i;
    int def_baudrate[36] = {50,75,110,134,150,200,300,600,1200,1800,
                            2400,4800,9600,19200,38400,57600,115200,
                            230400,
                            B50,B75,B110,B134,B150,B200,B300,B600,
                            B1200,B1800,B2400,B4800,B9600,B19200,B38400,
                            B57600,B115200,B230400 };

    for(i=0; i<18; i++)
        if(baud == def_baudrate[i])
            baud = def_baudrate[i+18];
	
	if (((*sr_cfg).fd = open(dev, O_RDWR | O_NOCTTY)) == -1)/* | O_NDELAY*/
	{
		perror("serial_init():Can't open device ");
		return (*sr_cfg).fd;
	}

	(*sr_cfg).term_attr.c_cflag = 0;
	(*sr_cfg).term_attr.c_cflag = (baud | CS8  | CLOCAL | CREAD) & (~CSTOPB);
	(*sr_cfg).term_attr.c_iflag = 0;/*IGNPAR;*/
	(*sr_cfg).term_attr.c_oflag = 0;
	(*sr_cfg).term_attr.c_lflag = 0;

	if (tcsetattr((*sr_cfg).fd, TCSAFLUSH, &(*sr_cfg).term_attr) != 0)
        {perror("serial_init():tcsetattr() failed");}
	
	/*read(..) kehrt umgehen zum Prog. zurück wenn nicht gesendet wird*/
	/*fcntl((*sr_cfg).fd, F_SETFL, FNDELAY);*/

	return (*sr_cfg).fd;
}
/*Serielle Schnittstelle schliessen und freigeben*/
void serial_relase(struct serial_cfg *sr_cfg)
{
	close((*sr_cfg).fd);
}
/*Ein Byte senden*/
int send_byte(unsigned char byte,struct serial_cfg *sr_cfg)
{
	int datacount = 0;
    if(sr_cfg->fd > -1)
	   datacount = write((*sr_cfg).fd,&byte,1);
	return datacount;
}
/*Das beginn Byte senden und anschliessend in size angegebene Anzahl
 von Bytes aus dem buffer senden*/
int send_data(unsigned char *buffer, int size, struct serial_cfg *sr_cfg)
{
    int count = 0;
    if(sr_cfg->fd > -1)
        count = write((*sr_cfg).fd,buffer,size);
    return count;
}
/*Das beginn Byte senden und anschliessend in size angegebene Anzahl
 von Bytes aus dem buffer senden*/
int send_data_bc(unsigned char *buffer,unsigned char beginn,int size,struct serial_cfg *sr_cfg)
{
	int count = 0;
    if(sr_cfg->fd > -1) {
	   write((*sr_cfg).fd,&beginn,1);
	   count = write((*sr_cfg).fd,buffer,size);
    }
	return count;
}
/*Ein Byte empfangen*/
unsigned char reciv_byte(struct serial_cfg *sr_cfg)
{
	unsigned char reciv_byte = 0;
	FD_ZERO(&(*sr_cfg).input_fdset);
	FD_SET((*sr_cfg).fd, &(*sr_cfg).input_fdset);

	if (select((*sr_cfg).fd+1, &(*sr_cfg).input_fdset, NULL, NULL, NULL)== -1) 
        perror("reciv_byte():select() failed");
	else
    {
		if (FD_ISSET((*sr_cfg).fd, &(*sr_cfg).input_fdset)) 
        {
			if (read((*sr_cfg).fd,&reciv_byte,1) == -1)
				perror("reciv_byte():read() failed");
		}
		else
			perror("reciv_byte():FD_ISSET failed");	
	}
	return reciv_byte;
}
/*In size angegebene Anzahl von Zeichen empfangen und im buffer schreicben*/
int reciv_data(unsigned char *buffer,int size,struct serial_cfg *sr_cfg)
{
	int count=0;
	buffer[0]=0;

	FD_ZERO(&(*sr_cfg).input_fdset);
	FD_SET((*sr_cfg).fd, &(*sr_cfg).input_fdset);

	if (select((*sr_cfg).fd+1, &(*sr_cfg).input_fdset, NULL, NULL, NULL)== -1) 
        perror("reciv_data():select() failed");
	else
	{	
		if (FD_ISSET((*sr_cfg).fd, &(*sr_cfg).input_fdset))
		{
			if ((count=read((*sr_cfg).fd,buffer,size)) == -1)
				perror("reciv_data():read() failed");
		}
		else
			perror("reciv_data():FD_ISSET failed");		
	}	
	return count;
}
/*So lange warten bis in byte angegebenes Zeichen empfangen wurde*/
unsigned char wait_byte(unsigned char byte,struct serial_cfg *sr_cfg)
{
    unsigned char reciv_byte = 0;

    while(reciv_byte != byte)
    {
        FD_ZERO(&(*sr_cfg).input_fdset);
        FD_SET((*sr_cfg).fd, &(*sr_cfg).input_fdset);

        if (select((*sr_cfg).fd+1, &(*sr_cfg).input_fdset, NULL, NULL, NULL)== -1) 
            perror("wait_byte():select() failed");
        else
        {
            if (FD_ISSET((*sr_cfg).fd, &(*sr_cfg).input_fdset))
            {
                if (read((*sr_cfg).fd,&reciv_byte,1) == -1)
                    perror("wait_byte():read() failed");
            }
            else
                 perror("wait_byte():FD_ISSET failed");
        }
    }
    return reciv_byte;
}
/*Anzahl der Bytes im Buffer*/
int inputbuf_bytes(struct serial_cfg *sr_cfg)
{
    int bytes;
    if(sr_cfg != NULL)
        ioctl(sr_cfg->fd, FIONREAD, &bytes);
    else
        perror("inputbuf_bytes():sr_cfg == NULL");
    return bytes;
}

/*Status der Steuer-Leitungen setzen*/
void set_rts(bool status, struct serial_cfg *sr_cfg)
{
    int temp;

    if(sr_cfg != NULL)
    {
        ioctl(sr_cfg->fd, TIOCMGET, &temp);
        if(status)
        {
            temp |= TIOCM_RTS;
            ioctl(sr_cfg->fd, TIOCMSET, &temp);
        }
        else
        {
            temp &= ~TIOCM_RTS;
            ioctl(sr_cfg->fd, TIOCMSET, &temp);
        }
    }
    else
        perror("set_rts():sr_cfg == NULL");
}
/****/
void set_dtr(bool status, struct serial_cfg *sr_cfg)
{
    int temp;

    if(sr_cfg != NULL)
    {
        ioctl(sr_cfg->fd, TIOCMGET, &temp);
        if(status)
        {
            temp |= TIOCM_DTR;
            ioctl(sr_cfg->fd, TIOCMSET, &temp);
        }
        else
        {
            temp &= ~TIOCM_DTR;
            ioctl(sr_cfg->fd, TIOCMSET, &temp);
        }
    }
    else
        perror("set_dtr():sr_cfg == NULL");
}
/*Status der Steuer-Leitungen abfragen*/
bool get_dcd(struct serial_cfg *sr_cfg)
{
    bool rc;
    int status;

    if(sr_cfg != NULL)
    {
        ioctl(sr_cfg->fd, TIOCMGET, &status);
        rc = (bool)(status  & TIOCM_CAR);
    }
    else
    {
        perror("get_dcd():sr_cfg == NULL");
        rc = false;
    }
    return rc;
}
/****/
bool get_dtr(struct serial_cfg *sr_cfg)
{
    bool rc;
    int status;

    if(sr_cfg != NULL)
    {
        ioctl(sr_cfg->fd, TIOCMGET, &status);
        rc = (bool)(status  & TIOCM_DTR);
    }
    else
    {
        perror("get_dtr():sr_cfg == NULL");
        rc = false;
    }
    return rc;
}
/****/
bool get_dsr(struct serial_cfg *sr_cfg)
{
    bool rc;
    int status;

    if(sr_cfg != NULL)
    {
        ioctl(sr_cfg->fd, TIOCMGET, &status);
        rc = (bool)(status  & TIOCM_DSR);
    }
    else
    {
        perror("get_dsr():sr_cfg == NULL");
        rc = false;
    }
    return rc;
}
/****/
bool get_rts(struct serial_cfg *sr_cfg)
{
    bool rc;
    int status;

    if(sr_cfg != NULL)
    {
        ioctl(sr_cfg->fd, TIOCMGET, &status);
        rc = (bool)(status  & TIOCM_RTS);
    }
    else
    {
        perror("get_rts():sr_cfg == NULL");
        rc = false;
    }
    return rc;
}
/****/
bool get_cts(struct serial_cfg *sr_cfg)
{
    bool rc;
    int status;

    if(sr_cfg != NULL)
    {
        ioctl(sr_cfg->fd, TIOCMGET, &status);
        rc = (bool)(status  & TIOCM_CTS);
    }
    else
    {
        perror("get_cts():sr_cfg == NULL");
        rc = false;
    }
    return rc;
}
/****/
bool get_ri(struct serial_cfg *sr_cfg)
{
    bool rc;
    int status;

    if(sr_cfg != NULL)
    {
        ioctl(sr_cfg->fd, TIOCMGET, &status);
        rc = (bool)(status  & TIOCM_RI);
    }
    else
    {
        perror("get_dtr():sr_cfg == NULL");
        rc = false;
    }
    return rc;
}
/*Uebersetzungsdatum*/
char *serialport_cdate()
{
        static char *date =__DATE__ "  " __TIME__;
        return date;
}
/*Version der Bibliothek*/
char *serialport_version()
{
        static char *date ="1.1.0";
        return date;
}


/*************
 * ENDE
 *************/
