/**************************************************************
 * serialport.h
 * Funktionen fuer den Datentransfer ueber serielle Schnitstelle
 * ehemals pc_mcu.h
 **************************************************************/
#include <stdbool.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>

/**************************************************************
 **                         Makros                           **
 **************************************************************/
#define  __SERIALPORT__   1

/**************************************************************
 **                         Strukturen                       **
 **************************************************************/
struct serial_cfg
{
    int fd;
    fd_set input_fdset;
    struct termios term_attr;
};
/**************************************************************
 **                     Funktionen                           **
 **************************************************************/

/*Serielle Schnittstelle offnen*/
int serial_init(char *dev, int baud, struct serial_cfg *sr_cfg);
/*Serielle Schnittstelle schliessen und freigeben*/
void serial_relase(struct serial_cfg *sr_cfg);
/*Ein Byte senden*/
int send_byte(unsigned char byte, struct serial_cfg *sr_cfg);
/*Die in size angegebene Anzahl von Bytes aus dem buffer senden*/
int send_data(unsigned char *buffer, int size, struct serial_cfg *sr_cfg);
/*Das beginn Byte senden und anschliessend in size angegebene Anzahl
 von Bytes aus dem buffer senden*/
int send_data_bc(unsigned char *buffer,unsigned char beginn,int size,
                                             struct serial_cfg *sr_cfg);
/*Ein Byte empfangen*/
unsigned char reciv_byte(struct serial_cfg *sr_cfg);
/*In size angegebene Anzahl von Zeichen empfangen und im buffer schreicben*/
int reciv_data(unsigned char *buffer,int size,struct serial_cfg *sr_cfg);
/*So lange warten bis in byte angegebenes Zeichen empfangen wurde*/
unsigned char wait_byte(unsigned char byte,struct serial_cfg *sr_cfg);
/*Anzahl der Bytes im Buffer*/
int inputbuf_bytes(struct serial_cfg *sr_cfg);
/*Uebersetzungsdatum*/
char *serialport_cdate();
/*Version der Bibliothek*/
char *serialport_version();
/*Status der Steuer-Leitungen setzen*/
void set_rts(bool status, struct serial_cfg *sr_cfg);
void set_dtr(bool status, struct serial_cfg *sr_cfg);
/*Status der Steuer-Leitungen abfragen*/
bool get_dcd(struct serial_cfg *sr_cfg);
bool get_dtr(struct serial_cfg *sr_cfg);
bool get_dsr(struct serial_cfg *sr_cfg);
bool get_rts(struct serial_cfg *sr_cfg);
bool get_cts(struct serial_cfg *sr_cfg);
bool get_ri(struct serial_cfg *sr_cfg);
